/*
 // @file		ImapWindowController.mm
 //
 // @version		Mailit for C/C++ 1.0 sample file
 //
 // @author		(C) 2007 Dacons LLP.
 //
 // @bug			If you have problems or suggestions, please contact
 //				the Online Support at: http://www.dacons.net/support
 */

#import "ImapWindowController.h"

#include PCH_FILE_NAME
#import "stringext.h"
#include <EmaiMessage.hpp>
#include "ImapFolder.hpp"
#include "ImapMessageData.hpp"
#include "MessageComposer.hpp"
#include "MessagePartEnumerator.hpp"
#include <assert.h>
#include <iostream>
#include "ErrorsDescriptions.hpp"


@implementation ImapWindowController

- (ImapFolder*)getActiveMailbox
{
	int selectedRow = mLastSelectedMailbox;
	if ((mImapResponseHandler->GetFoldersCount() > 0) && (selectedRow != -1))
	{
		Emai::UniString name = mImapResponseHandler->GetFolderAtIndex(selectedRow)->GetName();
		if (name.length() > 0)
		{
			ImapFolder* folder = mImapResponseHandler->FindFolderByName(name.c_str());
			return folder;
		}
	}
	return NULL;
}
  
- (void)setStatusText:(NSString *)inText
{
	[mStatusText setStringValue:inText];
	[mStatusPercent setStringValue:@""];
	[mStatusPercent displayIfNeeded];
	[mStatusText displayIfNeeded];
}

- (void)resetConnection
{
	[mMailServer setEnabled:TRUE];
	[mAccount setEnabled:TRUE];
	[mPassword setEnabled:TRUE];
	[mPort setEnabled:TRUE];
	[mUseSsl setEnabled:TRUE];
	[mAuthType setEnabled:TRUE];
	[mGetMailboxes setEnabled:TRUE];
	[mLicKey setEnabled:TRUE];
	[mAnotherServer setEnabled:FALSE];
	delete mImapSession;
	mImapSession = NULL;
	mImapResponseHandler->ClearFolderList();
	mLastSelectedMailbox = -1;
	[mMessages reloadData];
	[mMailboxes reloadData];
	[self setStatusText:@""];
}

-(void) showAlertForException:(Emai::Exception&) inException
{	
	NSAlert *alert = [NSAlert alertWithMessageText:@"An error occured while communicating with imap server"
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", errorsDescriptions[::abs(inException.GetErrorCode())]];
	[alert setAlertStyle:NSCriticalAlertStyle];
	[alert runModal];
}

-(void) showAlertWithMessageText:(NSString*) inInfText description:(NSString*) inMsgText
{
	NSAlert *alert = [NSAlert alertWithMessageText:inInfText
									 defaultButton:@"OK" alternateButton:NULL otherButton:NULL
						 informativeTextWithFormat:@"%@", inMsgText];
	[alert setAlertStyle:NSWarningAlertStyle];
	[alert runModal];
}

- (bool)fetchDataWithAttributes:(EmaiFetchMessageAttribute[])inFetchAttr
		attributesCount:(EmaiUint32)inAttributesCount
		forMessageSequence:(Emai::ImapSequenceSet&)inMsgSequence
{
	try
	{		
		EmaiFetchMessageData fetchData = {EmaiStructureVersionFirst};
		fetchData.selector = EmaiDataFetchAttr;
		fetchData.param.attr.attributeCount = inAttributesCount;
		fetchData.param.attr.attributes = inFetchAttr;
		
		// Fetch messages from server
		mImapSession->Fetch(inMsgSequence, &fetchData);
		[self setStatusText:@""];
		return true;
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		[self resetConnection];
		// Error formatting
		[self showAlertForException:exception];		
		return false;
	}
}

- (bool)connectSession
{
	try
	{
		if (mImapSession == NULL)
		{
			EmaiOptions options = EmaiNullOptions;
			if ([mUseSsl state]) 
				options = EmaiImapUseSSLOption;
			
			mImapSession = new Emai::ImapSession(mImapResponseHandler,
				*mStatusCallback, 60, options);
			
			int perc = 0;
			mImapSession->SetUserData(ImapCurrentPercent, &perc, sizeof(EmaiUint32));
			mImapSession->SetUserData(ImapPercentText, &mStatusPercent, sizeof(NSTextField*));
			mImapSession->SetUserData(ImapStatusText, &mStatusText, sizeof(NSTextField*));
				
			// Connect to server
			mImapSession->Connect([[mMailServer stringValue] uniString], [mPort intValue]);
			
			// Authenticate user
			EmaiUint32 authTypes[] = {
				EmaiIMAPAuthAutomatic,
				EmaiIMAPAuthLogin,
				EmaiIMAPAuthPlain,
				EmaiIMAPAuthCramMD5,
				EmaiIMAPAuthNTLM,
				EmaiIMAPAuthLoginCommand
			};
			
			mImapSession->Authenticate([[mAccount stringValue] uniString],
									   [[mPassword stringValue] uniString],
									   authTypes[[mAuthType indexOfSelectedItem]]);
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		[self resetConnection];
		
		// Error formatting
		[self showAlertForException:exception];
		return false;
	}
	return true;
}

- (id)init
{
	self = [super initWithWindowNibName:@"Window"];
	if (self == NULL)
		return self;
	mAttachmentsList = [[NSMutableArray alloc] init];
	mStatusCallback = new ImapCallback;
	mImapResponseHandler = new ImapResponseHandler;
	mLastSelectedMailbox = -1;
	[self window];
	[mMessages setDoubleAction:@selector(messageDoubleClick:)];
	
	return self;
}

- (void)dealloc
{
	[mAttachmentsList release];
	[mNoopTimer invalidate];
	delete mStatusCallback;
	delete mImapResponseHandler;
	delete mImapSession;
	[super dealloc];
}

- (void) windowDidLoad
{
	[mMailServer setStringValue:@"mail.server.net"];
	[mAccount setStringValue:@"your.account"];
	[mPassword setStringValue:@"password"];
	[mPort setIntValue:EmaiDefaultImapPort];
	
	// Set timer for sending noop every 60 seconds
	mNoopTimer = [[NSTimer scheduledTimerWithTimeInterval:
						60
                        target:self
                        selector:@selector(sendNoop:)
                        userInfo:nil
                        repeats:YES]
                        retain];
	if (mNoopTimer == nil)
		[NSApp terminate:nil];
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
	if (([notification object] == mMailboxes) && (mImapResponseHandler->GetFoldersCount() > 0))
	{
		mLastSelectedMailbox = [[notification object] selectedRow];
		ImapFolder* folder = [self getActiveMailbox];
		assert(folder != NULL);
		if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
		{
			mImapSession->SelectMailbox(folder->GetName().c_str());
			[self setStatusText:@""];
		}
		
		[mMessages reloadData];
	}
}

- (void)sendNoop:(NSTimer*)theTimer
{
	try
	{
		if (mImapSession != NULL)
		{
			mImapSession->Noop(); //Update message list
			if ([self getActiveMailbox] != NULL)
			{
				[mMailboxes reloadData];
				[mMessages reloadData];
			}
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		[self resetConnection];
		[self showAlertForException:exception];
	}
}

- (IBAction)addMailbox:(id)sender
{
	if (mImapSession != NULL)
		[[NSApplication sharedApplication] beginSheet:mInputWnd 
									   modalForWindow:[self window] modalDelegate:self 
									   didEndSelector:@selector(addMailboxDidEnd:returnCode:contextInfo:) 
									   contextInfo:NULL];
}

- (IBAction)sheetOK:(id)sender
{
	[[NSApplication sharedApplication] endSheet:[sender window] returnCode:NSAlertDefaultReturn];
}

- (IBAction)sheetCancel:(id)sender
{
	[[NSApplication sharedApplication] endSheet:[sender window] returnCode:NSAlertAlternateReturn];
}

- (void)addMailboxDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	if (returnCode == NSAlertDefaultReturn)
	{
		try
		{
			NSString *folderName = [mMailboxName stringValue];
			ImapFolder* folder = mImapResponseHandler->FindFolderByName([folderName uniString]);
			if (folder == NULL)
			{
				/*Create the folder*/
				mImapSession->CreateMailbox([folderName uniString]);
				mImapSession->ListMailboxes([[NSString stringWithString:@""] uniString],
											[folderName uniString]);
				[mMailboxes reloadData];
				[self setStatusText:@""];
			}
			else
			{
				[self showAlertWithMessageText:@"An error occured while creating new mailbox" 
								   description:[NSString stringWithFormat:@"Folder with name \"%@\" already exist", folderName]];
					
			}
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
}

- (IBAction)addMessage:(id)sender
{
	if (mImapSession != NULL)
		[[NSApplication sharedApplication] beginSheet:mMsgWnd modalForWindow:[self window]
					modalDelegate:self didEndSelector:@selector(addMessageDidEnd:returnCode:contextInfo:)
										  contextInfo:NULL];
}

- (void)addMessageDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	if (returnCode == NSAlertDefaultReturn)
	{
		NSString * message = NULL;
		if ([[mTo stringValue] length] == 0)
			message = @"You must enter recipient address.";
		else if ([[mFrom stringValue] length] == 0)
			message = @"You must enter sender address.";
		if (message != NULL)
		{
			[self showAlertWithMessageText:@"An error occured while adding new message" 
							   description:message];
			[[NSApplication sharedApplication] beginSheet:mMsgWnd modalForWindow:[self window]
											modalDelegate:self didEndSelector:@selector(addMessageDidEnd:returnCode:contextInfo:)
											contextInfo:NULL];
		}
		else
		{
			try
			{
				/*Check if the folder is present at the list*/
				ImapFolder* folder = [self getActiveMailbox];
				assert(folder != NULL);
				/*Compose the message*/
				/*Create the message composer*/
				MessageComposer messageComposer;

				/*Fix trouble with bare newlines*/
				NSMutableString* fixedStr = [NSMutableString stringWithString:[mMsgText string]];
				NSRange range = NSMakeRange(0, [fixedStr length]);
				[fixedStr replaceOccurrencesOfString:@"\n" withString:@"\r\n" options:NSCaseInsensitiveSearch range:range];
				
				/*Add the message text part*/
				messageComposer.SetUnicodeText([fixedStr uniString]);

				/*Add contact fields to the message*/
				Emai::ContactList contactTo;
				contactTo.AddContact([[mTo stringValue] uniString], NULL);
				Emai::ContactList contactFrom;
				contactFrom.AddContact([[mFrom stringValue] uniString], NULL);

				Emai::Message message = messageComposer.ComposeMessage(contactFrom, NULL, contactTo,
					NULL, NULL, [[mSubject stringValue] uniString]);

				if (message != NULL)
				{
					/*Add the message to the folder*/
					mImapSession->AppendMessage(folder->GetName().c_str(), NULL, NULL, message);
				}
				[self setStatusText:@""];
				[mMessages reloadData];
			}
			catch (Emai::Exception& exception)
			{
				//reset IMAP session object
				[self setStatusText:@""];
				[self showAlertForException:exception];
			}
		}
	}
}

- (IBAction)deleteMailbox:(id)sender
{
	if ((mImapSession != NULL) && ([mMailboxes numberOfSelectedRows] > 0))
	{
		NSAlert *alert = [NSAlert alertWithMessageText:@"Do you really want to delete the selected mailbox?"
							defaultButton:@"Delete" alternateButton:NULL otherButton:@"Cancel"
							informativeTextWithFormat:@"There is no undo for this operation."];
		[alert setAlertStyle:NSWarningAlertStyle];
		[[[alert buttons] objectAtIndex:0] setKeyEquivalent:@""];
		[[[alert buttons] objectAtIndex:1] setKeyEquivalent:@"\r"];

		[alert beginSheetModalForWindow:[self window] modalDelegate:self
							didEndSelector: @selector(sheetDidMailboxShouldDelete:returnCode:contextInfo:)
							contextInfo:NULL];
	}
}

- (void)sheetDidMailboxShouldDelete: (NSWindow *)sheet
						returnCode: (int)returnCode
						contextInfo: (void *)contextInfo
{
    if (returnCode == NSAlertDefaultReturn)
	{
		try
		{
			ImapFolder* folder = [self getActiveMailbox];
			assert(folder != NULL);
			/*Delete the folder*/
			[mMailboxes reloadData]; // we can not delete mailbox and rename together
			mImapSession->DeleteMailbox(folder->GetName().c_str());
			mImapResponseHandler->DeleteFolderFromFolderList(folder->GetName().c_str());
			[mMailboxes reloadData];
			[self setStatusText:@""];
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
}

- (IBAction)deleteMessage:(id)sender
{
	if ((mImapSession != NULL) && ([mMessages numberOfSelectedRows] > 0))
	{
		NSAlert *alert = [NSAlert alertWithMessageText:@"Do you really want to mark selected message(s) as deleted?"
		defaultButton:@"Mark" alternateButton:NULL otherButton:@"Cancel"
		informativeTextWithFormat:@"For removing deleted marks press Undelete button."];
		[alert setAlertStyle:NSWarningAlertStyle];
		[[[alert buttons] objectAtIndex:0] setKeyEquivalent:@""];
		[[[alert buttons] objectAtIndex:1] setKeyEquivalent:@"\r"];

		[alert beginSheetModalForWindow:[self window] modalDelegate:self
					 didEndSelector: @selector(sheetDidMessageShouldDelete:returnCode:contextInfo:)
					 contextInfo:NULL];

	}
}

- (void)sheetDidMessageShouldDelete: (NSWindow *)sheet
						returnCode: (int)returnCode
					    contextInfo: (void *)contextInfo
{
	if (returnCode == NSAlertDefaultReturn)
	{
		try
		{
			ImapFolder* folder = [self getActiveMailbox];
			assert(folder != NULL);
			
			EmaiStoreMessageData storeData = {EmaiStructureVersionFirst};
			storeData.selector = EmaiDataStoreAttr;
			storeData.param.attr.options = EmaiStoreAddFlags;
			storeData.param.attr.flags.flags = EmaiDeletedFlag;
			storeData.param.attr.flags.customFlags = NULL;
			
			/*Select the current message*/
			Emai::ImapSequenceSet messageSequenseUid;
			NSIndexSet* indexSet = [mMessages selectedRowIndexes];
			EmaiUint32 index = [indexSet firstIndex];
			while (index != NSNotFound)
			{
				messageSequenseUid.AddNumber(folder->GetMessageDataAtIndex(index + 1)->GetUid());
				index = [indexSet indexGreaterThanIndex:index];
			}
			
			
			//folder->SetSelectedForDeletion(messageSequenseUid);
			if (folder->GetSelectedForDeletion() == NULL)
				folder->SetSelectedForDeletion(messageSequenseUid);
			else
				folder->GetSelectedForDeletion().AddSequenceSet(messageSequenseUid);
			
			mImapSession->Store(messageSequenseUid, &storeData, EmaiImapUIDOption);
			[self setStatusText:@""];
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
}

- (IBAction)getMailboxes:(id)sender
{
	if (EmaiRegisterLibrary([[mLicKey stringValue] uniString]) == EmaiErrNoErr)
	{
		try
		{
			if (mImapSession != NULL)
			{
				delete mImapSession;
				mImapSession = NULL;
			}
			
			if (![self connectSession])
				return;

			mImapResponseHandler->ClearFolderList();

			/*Create the list of the folders*/
			mImapSession->ListMailboxes([[NSString stringWithString:@""] uniString],
									[[NSString stringWithString:@"*"] uniString]);
			mLastSelectedMailbox = 0;
			
			[mMailboxes reloadData];
			[mMailServer setEnabled:FALSE];
			[mAccount setEnabled:FALSE];
			[mPassword setEnabled:FALSE];
			[mPort setEnabled:FALSE];
			[mUseSsl setEnabled:FALSE];
			[mAuthType setEnabled:FALSE];
			[mAnotherServer setEnabled:TRUE];
			[sender setEnabled:FALSE];
			[mLicKey setEnabled:FALSE];
			[self setStatusText:@""];
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			delete mImapSession;
			mImapSession = NULL;
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
	else
		[self showAlertWithMessageText:@"Licence key is invalid"
						   description:@"Restart application and enter valid key"];}

- (int)numberOfRowsInTableView:(NSTableView *)aTable
{
	if (aTable == mMailboxes)
		return (int)mImapResponseHandler->GetFoldersCount();
	else if (aTable == mMessages)
	{
		ImapFolder* folder = [self getActiveMailbox];
		if ((folder != NULL) && ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute))
			return folder->GetExists();
	}
	else
		return [mAttachmentsList count];
	
	return 0;
}

- (id)tableView:(NSTableView *)aTable 
	 objectValueForTableColumn:(NSTableColumn *)aCol row:(int)aRow
{
	// Fill tables with data
	if (aTable == mMailboxes)
	{
		return [NSString stringWithCharacters:mImapResponseHandler->GetFolderAtIndex(aRow)->GetName().c_str()
									   length:mImapResponseHandler->GetFolderAtIndex(aRow)->GetName().length()];
		
	}
	else if (aTable == mMessages)
	{
		ImapFolder* folder = [self getActiveMailbox];
		if (folder != NULL)
		{
			if (folder->GetExists() > 0)
			{
				// Fill message list with message from current folder
				bool fetchResult = true;
				if (folder->GetMessageDataAtIndex(aRow + 1) == NULL)
				{
					Emai::ImapSequenceSet msgSequence;
					msgSequence.AddNumber(aRow + 1);
					EmaiFetchMessageAttribute attr[] = {{EmaiFetchFlags}, {EmaiFetchRFC822Header}, {EmaiFetchUid}};
					
					fetchResult = [self fetchDataWithAttributes:attr attributesCount:3 forMessageSequence:msgSequence];
				}
				if (folder->GetMessageDataAtIndex(aRow + 1) != NULL)
				{
					Emai::Message message = folder->GetMessageDataAtIndex(aRow + 1)->GetMessage();
					if (message.GetField(NULL, EmaiSubject) == NULL)
						return @"";
					else
					{
						Emai::UniString subject = message.GetField(NULL, EmaiSubject);
						return [NSString stringWithCharacters:subject.c_str() length:subject.length()];
					}
				}
			}
		}
	}
	else if (aTable == mSendAttachments)
		return [mAttachmentsList objectAtIndex:aRow];
	return @"";
}

- (void)tableView:(NSTableView *)aTable willDisplayCell:(id)aCell 
   forTableColumn:(NSTableColumn *)aCol row:(int)aRow
{
	ImapFolder* folder = NULL;
	if (aTable == mMailboxes)
	{
		folder = mImapResponseHandler->GetFolderAtIndex(aRow);
		assert(folder != NULL);
		if ((folder->GetAttributes() & EmaiStandardAttibutesMask) == EmaiNoselectAttribute)
			// Sets gray color for mailbox with EmaiNoselectAttribute attribute
			[aCell setTextColor:[NSColor grayColor]];
		else 
			// Sets black color for other mailboxes
			[aCell setTextColor:[NSColor blackColor]];
	}
	else if(aTable == mMessages)
	{
		folder = [self getActiveMailbox];
		if (folder != NULL)
		{
			if (folder->GetExists() > 0)
			{
				if (folder->GetMessageDataAtIndex(aRow + 1) != NULL)
				{
					if (folder->GetMessageDataAtIndex(aRow + 1)->GetFlags() & EmaiDeletedFlag)
					{
						if (folder->IsMarkedForDeletion(aRow + 1))
							[aCell setTextColor:[NSColor magentaColor]];
						else
							[aCell setTextColor:[NSColor redColor]];
					}
					else 
						[aCell setTextColor:[NSColor blackColor]];
				}
			}
		}
	}
}

- (void)fillTextField:(NSTextField*)ioField withFieldType:(EmaiOptions)inType 
		 usingEnumerator:(MessagePartEnumerator&)inEnumerator
{
			
	Emai::UniString name;
	Emai::UniString address;
	EmaiUint32 i = 0;
				
	Emai::ContactList contacts = inEnumerator.GetContacts(inType);
	EmaiUint32 count = contacts.GetCount();
	NSMutableString* str = [NSMutableString string];
	for (i = 0; i < count; ++i)
	{
		name = contacts.GetName(i);
		address = contacts.GetAddress(i);
		if ((name.length() > 0) && (address.length() > 0))
			[str appendString:[NSString stringWithFormat:@"%S - %S; ",
				name.c_str(), address.c_str()]];
		else if ((name.length() > 0) && (address.length() == 0))
			[str appendString:[NSString stringWithFormat:@"%S; ",
				name.c_str()]];
		else if ((name.length() == 0) && (address.length() > 0))
			[str appendString:[NSString stringWithFormat:@"%S; ",
				address.c_str()]];
	}
	[ioField setStringValue:str];
}

- (IBAction)messageDoubleClick:(id)sender
{
	try
	{
		if (![self connectSession])
			return;
	
		ImapFolder* folder = [self getActiveMailbox];
		assert(folder != NULL);
		if (index >= 0)
		{
			/*Select the folder. For broken connection.*/
			if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
				mImapSession->SelectMailbox(folder->GetName().c_str());
			
			EmaiUint32 messageNumber = [mMessages selectedRow] + 1;
			bool fetchResult = true;
			if (!folder->GetMessageDataAtIndex(messageNumber)->HasFullMessageData()) 
			{
				EmaiFetchMessageAttribute attr[] = {{EmaiFetchRFC822}};
				Emai::ImapSequenceSet msgSequence;
				msgSequence.AddNumber(messageNumber);
				fetchResult = [self fetchDataWithAttributes:attr attributesCount:1 forMessageSequence:msgSequence];
			}
			
			if (fetchResult)
			{
				Emai::Message message = folder->GetMessageDataAtIndex(messageNumber)->GetMessage();

				/*Create the message enumerator*/
				MessagePartEnumerator messagePartEnumerator(message);
				messagePartEnumerator.Enumerate();

				[self fillTextField:mSendFrom withFieldType:EmaiFrom 
					usingEnumerator:messagePartEnumerator];
				[self fillTextField:mSendTo withFieldType:EmaiTo 
					usingEnumerator:messagePartEnumerator];
				[self fillTextField:mSendCC withFieldType:EmaiCC 
					usingEnumerator:messagePartEnumerator];
				[mSubject2 setStringValue:[NSString stringWithCharacters:
					messagePartEnumerator.GetSubject().c_str()
					length:messagePartEnumerator.GetSubject().length()]];
							
				[mAttachmentsList removeAllObjects];
				EmaiUint32 attachmentsCount = messagePartEnumerator.GetAttachmentsCount();
				EmaiUint32 length = 0;
				if (attachmentsCount > 0)
				{
					Emai::UniString fileName;
					for (EmaiUint32 i = 0; i < attachmentsCount; i++)
					{
						messagePartEnumerator.GetAttachmentAtIndex(i, fileName, NULL, length);
						[mAttachmentsList addObject:[NSString stringWithFormat:
							@"%S (size = %d)", 
							fileName.c_str(), length]];
					}
				}

				attachmentsCount = messagePartEnumerator.GetInlinesCount();
				length = 0;
				Emai::UniString contentId;
				if (attachmentsCount > 0)
				{
					Emai::UniString fileName;
					for (EmaiUint32 i = 0; i < attachmentsCount; i++)
					{
						messagePartEnumerator.GetInlineAtIndex(i, fileName, contentId, NULL, length);
						[mAttachmentsList addObject:[NSString stringWithFormat:
							@"%S (size = %d content ID = %S)", 
							fileName.c_str(), length, contentId.c_str()]];
					}
				}

				Emai::UniString text;
				messagePartEnumerator.GetUnicodeText(text);
				[mSendMsgText setString:[NSString stringWithFormat:@"%S", 
					text.c_str()]];
				
				[mSendAttachments reloadData];
				[self setStatusText:@""];
				
				[[NSApplication sharedApplication] beginSheet:mSendMsgWnd modalForWindow:[self window]
												modalDelegate:self didEndSelector:@selector(addSendMsgDidEnd:returnCode:contextInfo:) 
												  contextInfo:NULL];
			}
		}
		else
		{
			[self showAlertWithMessageText:@"An error occured while readding message"
							description:[NSString stringWithFormat:@"There is no folder \"%S\" in mail account",
							folder->GetName().c_str()]];
		}
		[self setStatusText:@""];
	}
	catch (Emai::Exception& exception)
	{
		[self setStatusText:@""];
		[self showAlertForException:exception];
	}
}

- (void)addSendMsgDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
}

- (IBAction)expungeMessages:(id)sender
{
	if (mImapSession != NULL)
	{
		NSAlert *alert = [NSAlert alertWithMessageText:@"Do you really want to delete marked message(s) from server?"
										 defaultButton:@"Delete" alternateButton:NULL otherButton:@"Cancel"
							 informativeTextWithFormat:@"There is no undo for this operation."];
		[alert setAlertStyle:NSWarningAlertStyle];
		[[[alert buttons] objectAtIndex:0] setKeyEquivalent:@""];
		[[[alert buttons] objectAtIndex:1] setKeyEquivalent:@"\r"];
		
		[alert beginSheetModalForWindow:[self window] modalDelegate:self
						 didEndSelector: @selector(sheetDidMessageShouldExpunge:returnCode:contextInfo:)
							contextInfo:NULL];
	}
}

- (void)sheetDidMessageShouldExpunge: (NSWindow *)sheet
						 returnCode: (int)returnCode
					    contextInfo: (void *)contextInfo
{
	if (returnCode == NSAlertDefaultReturn)
	{
		try
		{
			ImapFolder* folder = [self getActiveMailbox];
			if (folder != NULL)
			{
				if (folder->GetSelectedForDeletion() != NULL)
				{
					/*Expunge all messages, which have been selected*/
					mImapSession->Expunge(folder->GetSelectedForDeletion());
					folder->ResetSelectedForDeletion();
					[mMessages reloadData];
				}
			}
			[self setStatusText:@""];
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}		
	}
}		
- (IBAction)setAnotherServer:(id)sender
{
	if (mImapSession != NULL)
		mImapSession->Disconnect();
	[self resetConnection];
}

- (void)tableView:(NSTableView *)aTable setObjectValue:(id)aData 
					forTableColumn:(NSTableColumn *)aCol row:(int)aRow
{
	if (aTable == mMailboxes)
	{
		try
		{
			ImapFolder* folder = mImapResponseHandler->GetFolderAtIndex(aRow);
			assert(folder != NULL);
			if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
			{
				NSString* newName = aData;
				if ([newName caseInsensitiveCompare:[NSString stringWithCharacters:folder->GetName().c_str()
																			length:folder->GetName().length()]] != NSOrderedSame)
				{
					mImapSession->RenameMailbox(folder->GetName().c_str(), [newName uniString]);
					mImapResponseHandler->RenameFoldersInFolderList(folder->GetName().c_str(), [newName uniString]);
					[mMailboxes reloadData];
					[self setStatusText:@""];
				}
			}
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
}

- (void)renameMailboxDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
	if (returnCode == NSAlertDefaultReturn)
	{
		try
		{
			ImapFolder* folder = [self getActiveMailbox];
			assert(folder != NULL);
			if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
			{
				NSString* newName = [mMailboxName stringValue];
				if ([newName caseInsensitiveCompare:[NSString stringWithCharacters:folder->GetName().c_str()
													length:folder->GetName().length()]] != NSOrderedSame)
				{
					mImapSession->RenameMailbox(folder->GetName().c_str(), [newName uniString]);
					mImapResponseHandler->RenameFoldersInFolderList(folder->GetName().c_str(), [newName uniString]);
					mImapSession->SelectMailbox([newName uniString]);
					[mMailboxes reloadData];
					[self setStatusText:@""];
				}
			}
			
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
}

- (IBAction)renameMailbox:(id)sender
{
	if (mImapSession != NULL)
		[[NSApplication sharedApplication] beginSheet:mInputWnd 
									   modalForWindow:[self window] modalDelegate:self 
									   didEndSelector:@selector(renameMailboxDidEnd:returnCode:contextInfo:) 
									   contextInfo:NULL];
}

- (IBAction)undeleteMessages:(id)sender
{
	if (mImapSession != NULL)
	{
		try
		{
			ImapFolder* folder = [self getActiveMailbox];
			assert(folder != NULL);
			EmaiStoreMessageData storeData = {EmaiStructureVersionFirst};
			storeData.selector = EmaiDataStoreAttr;
			storeData.param.attr.options = EmaiStoreRemoveFlags;
			storeData.param.attr.flags.flags = EmaiDeletedFlag;
			storeData.param.attr.flags.customFlags = NULL;
			
			Emai::ImapSequenceSet messageSequenseUid;
			NSIndexSet* indexSet = [mMessages selectedRowIndexes];
			
			EmaiUint32 index = [indexSet firstIndex];
			while (index != NSNotFound)
			{
				messageSequenseUid.AddNumber(folder->GetMessageDataAtIndex(index + 1)->GetUid());
				folder->GetMessageDataAtIndex(index + 1)->SetFlags(folder->GetMessageDataAtIndex(index + 1)->GetFlags() & ~EmaiDeletedFlag);
				index = [indexSet indexGreaterThanIndex:index];
			}
			
			if (messageSequenseUid.GetCount() > 0)
			{
				/*Select the current message*/
				folder->GetSelectedForDeletion().RemoveSequenceSet(messageSequenseUid);
				mImapSession->Store(messageSequenseUid, &storeData, EmaiImapUIDOption);
				[mMessages reloadData];
			}
			[self setStatusText:@""];
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			[self setStatusText:@""];
			[self showAlertForException:exception];
		}
	}
	
}
@end
