/*!
// @file		EmaiObject.hpp
//
// @brief		C++ wrappers for basic library objects
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIOBJECT_HPP__
#define __EMAIOBJECT_HPP__

#include <EmaiExceptions.hpp>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{

/*!
// @Object
// @brief EmaiObject wrapper class
*/
template <class ObjectType>
class Object
{
protected:
											Object();
											~Object();

											Object(
													ObjectType inObject,
													bool inAddRef);
											Object(
													const Object& inCopy);
	Object&									operator = (
													const Object& inCopy);

	ObjectType								mObject;

	typedef Object<ObjectType>				Parent;

public:

	/*!
	// @brief	Increments reference count value of the object by 1
	*/
	void									AddRef();

	/*!
	// @brief	Decreases reference count value of the object by 1. If reference
	//				count become zero the object is destroyed and memory occupied by the
	//				object is freed
	//				This function returns EmaiErrIncorrectParameter error if caller trys
	//				to destroy (set reference count to zero) message part objects which
	//				are owned by message object.
	*/
	void									Release();

	/*!
	// @brief	Returns reference count value of the object.
	//
	// @return	on successful function exit contains reference count
	//				value of the object
	*/
	EmaiUint32								RefCount();

	/*!
	// @brief	Allows to save user data to library object.
	//
	// @param	inKey - key used to identify the data
	// @param	inData - pointer to data to be copied into object. If inData is NULL
	//				the function will remove data from the object, inDataSize value
	//				is ignored in this case
	// @param	inDataSize - size of the data in bytes
	*/
	void									SetUserData(
													EmaiUint32 inKey,
													const void* inData,
													EmaiUint32 inDataSize);

	/*!
	// @brief	Allows to retrieve user data from library object stored in object
	//				earlier using EmaiObjectSetUserData.
	//
	// @param	inKey - key used to identify the data
	// @param	outData - pointer to memory where retrieved data should be stored.
	//				If NULL ioDataSize contains size of the data stored under key inKey
	// @param	ioDataSize - size of the outData block in bytes, on exit contains
	//				size of the data copied in outData (if non-NULL) or size of the data
	//				(if outData is NULL)
	*/
	void									GetUserData(
													EmaiUint32 inKey,
													void* outData,					/*!< can be NULL*/
													EmaiUint32& ioDataSize);

											operator ObjectType() const;
};


/*
// Object class implemetation
*/
template <class ObjectType>
Object<ObjectType>::Object(
		ObjectType inObject,
		bool inAddRef
	):
	mObject(inObject)
{
	if (inAddRef)
		AddRef();
}

template <class ObjectType>
Object<ObjectType>::Object():
	mObject(NULL)
{
}

template <class ObjectType>
Object<ObjectType>::Object(
		const Object<ObjectType>& inCopy
	):
	mObject(NULL)
{
	if (inCopy.mObject != NULL)
	{
		CheckErrorCode(EmaiObjectAddRef(inCopy.mObject));
		mObject = inCopy.mObject;
	}
}

template <class ObjectType>
Object<ObjectType>&
Object<ObjectType>::operator = (
		const Object<ObjectType>& inCopy
	)
{
	Release();
	mObject = NULL;
	
	if (inCopy.mObject != NULL)
		CheckErrorCode(EmaiObjectAddRef(inCopy.mObject));

	mObject = inCopy.mObject;
	return *this;
}

template <class ObjectType>
Object<ObjectType>::~Object()
{
	Release();
}

template <class ObjectType>
void
Object<ObjectType>::AddRef()
{
	if (mObject != NULL)
		CheckErrorCode(EmaiObjectAddRef(mObject));
}

template <class ObjectType>
void
Object<ObjectType>::Release()
{
	if (mObject != NULL)
		CheckErrorCode(EmaiObjectRelease(mObject));
}

template <class ObjectType>
EmaiUint32
Object<ObjectType>::RefCount()
{
	EmaiUint32 refCount = 0;
	CheckErrorCode(EmaiObjectGetRefCount(mObject, &refCount));
	return refCount;
}

template <class ObjectType>
void
Object<ObjectType>::SetUserData(
		EmaiUint32 inKey,
		const void* inData,
		EmaiUint32 inDataSize
	)
{
	CheckErrorCode(EmaiObjectSetUserData(mObject, inKey, inData, inDataSize));
}

template <class ObjectType>
void
Object<ObjectType>::GetUserData(
		EmaiUint32 inKey,
		void* outData,					/*can be NULL*/
		EmaiUint32& ioDataSize
	)
{
	CheckErrorCode(EmaiObjectGetUserData(mObject, inKey, outData, &ioDataSize));
}

template <class ObjectType>
Object<ObjectType>::operator ObjectType() const
{
	return mObject;
}

}


#endif /*__EMAIOBJECT_HPP__*/
